#********************************************************************************
# qware build environment - qube-binpkg                                         *
#                                                                               *
#  Author: Kai Peter, ©2019-YYYY                                                *
# Version: 0.2                                                                  *
# License: ISC                                                                  *
#                                                                               *
# Description: Create an archive of a binary distributable package.             *
#********************************************************************************

error() { printf "\033[1mError: %s\033[1m\n" "$1" ; exit; }

QRELEASEDIR            # placeholder for function chkRelDir (used multiple times)

mkBinDist() {
  cd "$SRCDIR" || error "Unable to change into dir $SRCDIR"
  make bindist || exit $?
  cd $TOPQUBEDIR
  return
}

getTopQubeDir                 # top level directory where we start any processing

CFGFILE=".qube/config"           # the config file HAVE TO exist in this location
[ ! -f "$CFGFILE" ] && error "No config file found!" || source "$CFGFILE"

# some default settings (fallback, usually set in config file)
[ ! $SRCDIR ] && SRCDIR="src"
[ ! -d "$SRCDIR" ] && error "Unable to determine sources directory!"

if [ ! "$TARGET" ] ; then
  [ -f $SRCDIR/PKGNAME.dist ] && TARGET=`head -1 "$SRCDIR/PKGNAME.dist"`; fi
[ ! $TARGET ] && error "Unable to determine a target name - aborting!"

[ ! -f $SRCDIR/VERSION ] && warning "No file VERSION found - using today's date." \
    && VERSION="$DATESTR" || VERSION=`head -1 $SRCDIR/VERSION`

TARGET="$TARGET-bin-$VERSION.tgz"
# do nothing if target exists in releases dir
[ -f "$RELEASESDIR/$TARGET" ] && \
  echo "A target $TARGET exists already!" && exit 2
# build target if not exists in source dir
[ ! -f "$SRCDIR/$TARGET" ] && mkBinDist

chkRelDir && mv $SRCDIR/$TARGET $RELEASESDIR && \
echo "Successful created binary tarball $TARGET in '$RELEASESDIR'!"
