#********************************************************************************
# qware build environment - qube-srcpkg                                         *
#                                                                               *
#  Author: Kai Peter ©2015-YYYY                                                 *
# Version: 2.1.0                                                                *
# License: ISC                                                                  *
#                                                                               *
# Description: Create a source code release.                                    *
#********************************************************************************
#---------- begin (standard) qware configuration section ------------------------
VERSION=`cat $0 | grep -m 1 'Version' | cut -d\   -f3`       # get version number
ME=`basename $0`

LOGDATE=`date +'%b %d %H:%M:%S'`
LOGFILE="QLOGDIR/${ME}.log"
DATESTR=`date +%Y%m%d`
LIBDIR="QLIBDIR"

[ $DBG ] || DBG=0                                # set debug level (dafault: off)

error() { printf "\033[1mError: %s\033[0m\n" "$1"; exit 1; }
warning() { printf "\033[93mWarning: %s\033[0m\n" "$1"; return; }
#---------- end qware configuration section -------------------------------------

QRELEASEDIR            # placeholder for function chkRelDir (used multiple times)
prepSources() {        # check and prepare the sources **************************
  TMPDIR=`mktemp -d tmp.XXXXXXXXXX` || \
    error "Unable to create temporary directory!"
  echo -n "Preparing sources ... "
  # set includes and excludes for rsync
  [ -f .qube/pkginclude ] && INCLUDE="--include-from .qube/pkginclude"
  [ -f .qube/pkgexclude ] && EXCLUDE="--exclude-from .qube/pkgexclude"
  # by convention we use a .gitignore file in the top level source dir
  [ -f $SRCDIR/.gitignore ] && EXCLUDE=" --exclude-from $SRCDIR/.gitignore"
  rsync -Lr $SRCDIR/* $TMPDIR $INCLUDE $EXCLUDE || exit $?
  echo "done"
}
#********************************************************************************
mkArchive() {       # create the archive ****************************************
  ARCHIVE="$TARGET-$VERSION".tgz
  # some time the VERSION file contains the target name, sometimes not
#  echo $ARCHIVE | grep $TARGET >DEVNULL || ARCHIVE="$TARGET-$VERSION".tgz
#  ARCHIVE=$TARGET.tgz

  echo -n "Creating archive ... "
  tar -czf $ARCHIVE $TMPDIR --transform s/"$TMPDIR"/"$TARGET"/ || exit $?
  sha256sum $ARCHIVE > $ARCHIVE.sha256 || exit $?
  echo "done"

#  [ "$RELEASESDIR" ] || RELEASESDIR="releases"
#  if [ ! -d "$RELEASESDIR" ] ; then
#    mkdir -p "$RELEASESDIR" || error "Unable to create dir '$RELEASESDIR'."
#    chmod 0775 $RELEASESDIR || error "Unable to chmod dir $RELEASESDIR."
#  fi
  chkRelDir
  mv $ARCHIVE* $RELEASESDIR || exit 1
  [ "$DBG" -lt 1 ] && rm -fr "$TMPDIR"              # clean up on low debug level
}
getTopQubeDir                 # top level directory where we start any processing

CFGFILE=".qube/config"           # the config file HAVE TO exist in this location
[ ! -f "$CFGFILE" ] && error "No config file found!" || source "$CFGFILE"

# some default settings (fallback, usually set in config file)
[ ! $SRCDIR ] && SRCDIR="src"
[ ! -d "$SRCDIR" ] && error "Unable to determine sources directory!"

if [ ! "$TARGET" ] ; then
  [ -f $SRCDIR/PKGNAME.dist ] && TARGET=`head -1 "$SRCDIR/PKGNAME.dist"`; fi
[ ! $TARGET ] && error "Unable to determine a target name - aborting!"

# optional
[ ! -f $SRCDIR/LICENSE ] && warning "No file LICENSE found - ignored."
[ ! -f $SRCDIR/VERSION ] && warning "No file VERSION found - using today's date." \
    && VERSION="$DATESTR" || VERSION=`head -1 $SRCDIR/VERSION`

rm -rf tmp.*        # remove old TMPDIR's
echo "Attempting to get changelog status ... "
$LIBDIR/qube-chglog
prepSources         # prepare the sources
mkArchive           # create archive
echo "Successful created source code tarball $ARCHIVE in '$RELEASESDIR'!"
