#!/config/sh                              # just and only for syntax highlighting
#********************************************************************************
# qware build environment - header file of the qube configure script            *
#                                                                               *
#  Author: Kai Peter, ©2020-present                                             *
# Version: 0.3                                                                  *
# License: ISC                                                                  *
#                                                                               *
# Description: Standard commands and functions of the qube configure process.   *
#              Only developers should adapt it to their package requirements.   *
#********************************************************************************

declare -r QUBEDIR="build.in"     # MUST match with the first line in 'configure'

cd `dirname $0`
make clean

# general configure error handling: print message given by "$1" and exit
error() { [ "$1" ] && [ -n "$1" ] && printf "\033[91m$1!\033[0m\n" && exit 1; }

getSystype() {
  echo -n "Checking system ... "
  local OS=`uname -s | tr '[A-Z]' '[a-z]'`
  echo "detected $OS!"
  echo -n "Checking architecture ... "
  local ARCH=`uname -m`
  echo "$OS:$ARCH" > systype
  echo "build on $ARCH!"
}

mkShebang() {
  QUBE_SHEBANG=`which env` || exit 1   # an error shouldn't happen, but who knows
  echo "#!$QUBE_SHEBANG sh" > ac-warn.sh     # output was formerly 'warn-auto.sh'
  echo "# !!! WARNING: This file was auto-generated. Do NOT edit!" >> ac-warn.sh
  # Hint: To use a different interpreter than 'sh' you can do something like this
  #       in a Makefile recipe (just an example to get the idea):
  #
  #       cat ac-warn.sh | sed 's/ sh$/ perl/g' | ...

  return
}

# get variables from 'conf-build' and corresponding 'conf-*' files **************
. ./conf-build         # 'conf-build' is required and must be present in any case
[ -f conf-prefix ] && PREFIX=`head -1 conf-prefix`      # maybe a packagers patch
[ -f conf-suffix ] && SUFFIX=`head -1 conf-suffix`      # maybe a packagers patch


. "$QUBEDIR"/get-layout
. "$QUBEDIR"/get-usrcreds
. "$QUBEDIR"/get-externals
. "$QUBEDIR"/mk-buildtools

[ ! "$SYSTYPE" ] && getSystype

#
[ -f "$QUBEDIR/.no-layout" ]    || getLayout
[ -f "$QUBEDIR/.no-users" ]     || [ -f ".no-users" ] || getUsrCreds
[ -f "$QUBEDIR/.no-externals" ] || chkExtRes
[ -f "$QUBEDIR/.no-builtools" ] || mkBuildTools



# The last step: create 'ac-warn.sh' always *************************************
# Usually 'ac-warn.sh is required in 99.9% of build cases anyway, otherwise could
# be used in the Makefile to check if the configure script was invoked already.
[ -f ac-warn.sh ] || mkShebang
